#define _XOPEN_SOURCE
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include "im-protocol.h"
#include "matchbox-keyboard-remote.h"
#define ICON_FILE "/usr/share/pixmaps/matchbox-keyboard.png"
#define MB_KEYBOARD "matchbox-keyboard" 
#define STARTUP_DELAY 1 // allow 1 second for matchbox-keyboard to start

static void show_about(void) {
	gtk_show_about_dialog (NULL,
                       "program-name", "mb-systray",
                       "title", "About mb-systray",
                       "copyright", "© James Budiono 2015. GNU GPL Version 3 or later.",
                       "comments", "System-tray applet to show/hide matchbox keyboard.",
                       NULL);
}

static void ensure_keyboard_is_running(char **mb_args) {
	if (WEXITSTATUS(system("pidof " MB_KEYBOARD "> /dev/null" ))!=0) {
		// matchbox-keyboard not running, start it
		g_spawn_async(NULL, mb_args, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL);
		//g_spawn_command_line_async(MB_KEYBOARD " -d", NULL);
		g_usleep(STARTUP_DELAY * G_USEC_PER_SEC);
	}	
}

static void leftclick(GtkStatusIcon *si, gpointer s) {
	ensure_keyboard_is_running(s);
	protocol_send_event (MBKeyboardRemoteToggle);
}

static void rightclick(GtkStatusIcon *si, guint b,guint a_t, gpointer s) {
	GtkMenu* menu = (GtkMenu*) s;
	gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL, b, a_t);
}

static GtkItemFactoryEntry entries[] = {
  { "/_About",   NULL, show_about,          0, "<StockItem>", GTK_STOCK_ABOUT },
  { "/_Quit",    NULL, gtk_main_quit, 0, "<StockItem>", GTK_STOCK_QUIT } };

int main(int argc, char *argv[]){
	GtkStatusIcon *si;	
	GtkMenu *menu;
	int argc_offset;
	int runonce=0;
	int cmd;
	char **mb_args;
	
	gtk_init (&argc, &argv);
	
	// evaluate command lines
	argc_offset=1;
	if (argc > 1) {
		if (strcmp(argv[1],"-h")==0 || strcmp(argv[1],"--help")==0) {
			g_print(
"Usage: mb-systray [--hide|--show|--toggle] [matchbox-keyboard-args]\n"
"  --hide/--show/--toggle will hide/show/toggle the keyboard and exit.\n"
"  Otherwise mb-systray will run as a systray icon.\n"
"  The rest of the args are used as args for launching the keyboard.\n"
);
			exit(0);
		} else if (strcmp(argv[1],"--hide")==0) {
			runonce = 1;
			argc_offset++;
			cmd = MBKeyboardRemoteHide;
		} else if (strcmp(argv[1],"--show")==0) {
			runonce = 1;
			argc_offset++;
			cmd = MBKeyboardRemoteShow;
		} else if (strcmp(argv[1],"--toggle")==0) {
			runonce = 1;
			argc_offset++;
			cmd = MBKeyboardRemoteToggle;
		}
	}
	
	// pass all args to MB_KEYBOARD
	char *p1 = g_strjoinv("|", &argv[argc_offset]);
	char *p2 = g_strconcat(MB_KEYBOARD,"|","-d","|",p1,NULL);
	mb_args = g_strsplit(p2,"|",0);
	//g_print("%s\n",g_strjoinv(" ",p3));
	g_free(p2);
	g_free(p1);
	
	if (runonce) {
		ensure_keyboard_is_running (mb_args);
		protocol_send_event (cmd);
		exit(0);
	}

	// menu
	GtkItemFactory* factory = gtk_item_factory_new(GTK_TYPE_MENU, "<Root>", NULL);
	gtk_item_factory_create_items(factory, 2, entries, NULL);
	menu = GTK_MENU(gtk_item_factory_get_widget(factory, "<Root>")); 
	
	// start tray icon
	si = gtk_status_icon_new_from_file(ICON_FILE);
	gtk_status_icon_set_tooltip_markup(si, "Show keyboard.");
	g_signal_connect(G_OBJECT(si), "activate", G_CALLBACK(leftclick), mb_args);
	g_signal_connect(G_OBJECT(si), "popup-menu", G_CALLBACK(rightclick), menu);
	
	gtk_main();
	return 0;
}
